/**	RC制御クラスヘッダ.	関口 浩司 	2009-04-20
* Copyright (c) 2009 ZMP Inc. All rights reserved. **/

/// シミュレーション用に改変　2012-5-21

#ifndef  RCCONTROL_H_
#define RCCONTROL_H_/* ドライブ情報構造体 */

//void sleep(int t);
//void system(char *text);
// シミュレーション用関数
extern double dmGetTimeSimu();
extern void dmGetRealPosition(double *p, double *heading);

namespace zmp
{
namespace zrc
{

// Steering ServoMotor.hから一部抜粋
enum SERVO_INFO_ADR
{
    // 変更不可領域
    /**モデル番号 下位 byte. read only.
     */
    MODEL_NUMBER_L	 	= 0x00,
    /**モデル番号 上位 byte. read only.
     */
    MODEL_NUMBER_H 		= 0x01,
    /**	現在位置 下位byte.
     */
    PRESENT_POSITION_L	= 0x2a,
    /**	現在温度 上位byte.(1=10mv).
     */
    PRESENT_VOLTS_H		= 0x35,
};

struct DRIVE_VALUE
{
    char	id;                 /*** サーボID. */
    char	reverse;        /*** サーボ回転方向.*/
    char	baud_rate;      /*** 通信速度.*/
    char	return_delay;  /*** 返信遅延.*/
    float	cw_limit;        /*** CW 可動リミット. */
    float	ccw_limit;      /*** CCW 可動リミット. */
    int	temp_limit;    /*** 可動温度リミット.*/
    char	cw_comp_mrg;
    char	ccw_comp_mrg;
    char	cw_comp_slope;
    char	ccw_comp_slope;
    float	punch;                /*** Punch. */
    float	goal_position;       /*** 指定角度. */
    int	goal_time;            /*** 指定時間.*/
    char max_torque;          /*** 最大トルク.*/
    char torque_enable;       /*** トルク状態.*/
    float	present_position;   /*** 現在角度. */
    int	present_time;       /*** 現在時間.*/
    int	present_speed;     /*** 現在速度.*/
    int	present_current;        /*** 現在負荷(電流).*/
    int	present_temperature; /*** 現在温度.*/
    int	present_volts;           /*** 現在電圧.*/
};

/* 障害物センサ情報構造体 */
struct OBSTACLE_VALUE
{
    /*** 障害物センサ[8] [mm]. */
    unsigned short obstacle[8];
};

/*** 電源情報構造体 */
struct POWER_VALUE
{
    /***  モータ電流 [mA].*/
    double motor_current;
    /*** バッテリレベル [V].*/
    double battery_level;
};


/* 内部センサ情報構造体  */
struct SENSOR_VALUE
{
    float gyro;     /*** ジャイロセンサ [deg/sec].    */
    float acc_x;   /*** 加速度センサ(X軸) [G].        */
    float acc_y;   /*** 加速度センサ(Y軸) [G].        */
    float acc_z;   /*** 加速度センサ(Z軸) [G].        */
    float enc_0;  /*** モータ エンコーダ [cycle/sec].*/
    float enc_1;  /*** Wheel エンコーダ1 [mm/sec]. */
    float enc_2;  /*** Wheel エンコーダ2 [mm/sec]. */
    float enc_3;  /*** Wheel エンコーダ3 [mm/sec]. */
    float enc_4;  /*** Wheel エンコーダ4 [mm/sec]. */

};

/**
 * RCコントロールクラス.
 * RCをコントロールするためにBaseboardクラスへのアクセスを行う。
 */
class RcControl
{
public:
    RcControl();
    virtual ~RcControl();
    bool init();
    bool Start();
    bool Stop();
    bool Close();

    /*** 最大トルクの取得.
     * @param[out]	torque		最大トルク. 1％単位.
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetMaxTorque(char* torque);

    /*** 障害物センサ情報の取得要求.
     * @param[out]	value	障害物センサ情報が設定される構造体アドレス.
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetObstacleSensorInfoReq(OBSTACLE_VALUE* value);

    /*** 電源情報の取得要求
    * @param[out]	value	電源情報が設定される構造体アドレス.
    * @retval	true	情報取得成功.
    * @retval	false	情報取得失敗.
    */
    bool GetPowerInfoReq(POWER_VALUE* value);

    /*** サーボの現在位置の取得.
     * @param[out]	angle		present angle. 0.1度単位
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetPresentAngle(float* angle);

    /*** サーボの現在の回転速度取得.
     * @param[out]	speed	present speed. deg/sec単位.
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetPresentSpeed(int* speed);

    /*** センサ情報の取得要求.
     * @param[out]	value	センサ情報が設定される構造体アドレス.
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetSensorInfoReq(SENSOR_VALUE* value);

    /*** サーボ情報取得要求
     * BaseBoardに対してServo情報取得メッセージを送信して、現在保持しているServo情報を返す
     * @param[in]		id		サーボID.
     * @param[in]		adr		取得情報アドレス.
     * @param[in]		len		取得情報の長さ.
     * @param[out]	value	サーボ情報が設定される構造体アドレス.
     * @retval	true	情報取得成功.
     * @retval	false	情報取得失敗.
     */
    bool GetServoInfoReq(int id, SERVO_INFO_ADR adr, int len, DRIVE_VALUE* value);

    /** トルク状態の取得.
        * @param[out]	enable		トルク状態. 0=OFF,  1=ON,  2=BRAKE.
        * @retval	true	情報取得成功.
        * @retval	false	情報取得失敗.
        */
    bool GetTorqueEnable(char* enable);


    /*** 駆動速度の設定.
     * @param[in]		speed	駆動速度(mm/sec). 1mm単位で設定範囲は-2800～2800.
     * @retval	true	情報設定成功
     * @retval	false	情報設定失敗
     */
    bool SetDriveSpeed(int speed);

    /*** ジャイロオフセット.ジャイロセンサ出力のゼロ点設定.
	 * @param[in]		offset		ジャイロオフセット(deg/s).
	 */
	void SetGyroOffset(float offset);

    /*** 最大トルクの設定.
    * @param[in]		torque		最大トルク. 1%単位で設定範囲は0(0%)～100(100%).
    * @retval	true	情報設定成功
    * @retval	false	情報設定失敗
    */
    bool SetMaxTorque(int torque);

    /*** 電流オフセット設定.モータ電流のゼロ点設定
	 * @param[in]		offset		電流オフセット(A).
	 */
	void SetMotorCurrentOffset(float offset);

    /*** モータON/OFF設定要求.
     * @param[in]	enable		モータON/OFF.
     */
    void SetMotorEnableReq(char enable);

    /*** モータトルク設定要求.
     * @param[in]		p		P(比例動作).
     * @param[in]		i		I(積分動作).
     * @param[in]		d		D(微分動作).
     * @param[in]		w		width.
     */
    void SetMotorTorqueReq(char p, char i, char d, char w);


    /*** 通知情報フラグ設定要求.
    * @param[in]	flag		通知する情報を設定するフラグ. bit0=Sensor. bit1=Obstacle. bit2=Power. bit3～bit4=reserve.
    */
    void SetReportFlagReq(char flag);

    /*** 操舵角の設定.
     * @param[in]		angle	操舵角. 0.1度単位で設定範囲は-30.0～30.0.
     * @retval	true	情報設定成功
     * @retval	false	情報設定失敗
     */
    bool SetSteerAngle(float angle);

    /*** サーボモータのON/OFF.
     * @param[in]		enable		ON/OFF OFF=0, ON=1, BRAKE=2.
     * @retval	true	情報設定成功
     * @retval	false	情報設定失敗
     */
    bool SetServoEnable(int enable);



private:
    struct READ_STATE
    {
        bool	sensor;
        bool	obstacle;
        bool	power;
        bool	drive;
    };

    enum READ_KIND
    {
        READ_SENSOR,
        READ_OBSTACLE,
        READ_POWER,
        READ_DRIVE
    };
};
}
}
#endif /* RCCONTROL_H_ */
