#include "RoboCar.h"
#include <unistd.h>

RoboCar::RoboCar()
{
    // base bord init
    //system ("ipm_serial S 4 0");
    usleep(1000*1000);
    //system ("ipm_serial S 4 1");
    usleep(1000*1000);
    _RcControl.init ();
    _RcControl.Start ();
    // sensor init ... used gyro.enc.obs
    _RcControl.SetReportFlagReq (3);
    // actuator init ... ON
    _RcControl.SetMotorEnableReq (1);
    _RcControl.SetServoEnable (1);
    _RcControl.SetDriveSpeed (0);
    SteerAngle (0.0);
    // gyro offset
    gyro_offset = 0.0;
    const int OFFSET_CYCLE = 2000;
    for (int i = 0; i < OFFSET_CYCLE; i++)
    {
        memset (&sensor, 0, sizeof (SENSOR_VALUE));

        _RcControl.GetSensorInfoReq (&sensor);
        gyro_offset += sensor.gyro;
    }
    gyro_offset = gyro_offset / (double) OFFSET_CYCLE;
}

RoboCar:: ~RoboCar()
{
    _RcControl.SetDriveSpeed (0);
    _RcControl.SetSteerAngle (0.0);
    // 쓮d OFF
    _RcControl.SetMotorEnableReq (0);
    _RcControl.SetServoEnable (0);
    _RcControl.Stop ();
    _RcControl.Close ();
}

void RoboCar::EndRoboCar ()
{
    _RcControl.SetDriveSpeed (0);
    SteerAngle (0.0);
    _RcControl.SetMotorEnableReq (0);
    _RcControl.SetServoEnable (0);
    _RcControl.Stop ();
    _RcControl.Close ();
}

void RoboCar::Go (int speed)
{
    _RcControl.SetDriveSpeed (speed);
}

void RoboCar::SteerAngle (float angle)
{
    const float STEER_OFFSET= 53.0;
    _RcControl.SetSteerAngle (-angle + STEER_OFFSET);
}

void RoboCar::GetDeadReckoningData (double *x,double *y,double *theta,double time)
{
    // get data
    _RcControl.GetSensorInfoReq (&sensor);
    //_RcControl.GetServoInfoReq (1,
    //			PRESENT_POSITION_L,
    //			PRESENT_VOLTS_H - PREAENT_POSITION_L,
    //			&drive);
    double d_r = (sensor.enc_3 + sensor.enc_4) / 2.0;
    //double d_ang = drive.present_position - old_ang;
    //d_ang = d_ang * M_PI / 180.0;
    //d_x = d_r * cos(d_ang);
    //d_y = d_r * sin(d_ang);
    // differential angle

    const double GYRO_NOISE_CUT = 2.0;
    if (fabs (sensor.gyro) > GYRO_NOISE_CUT) d_theta = sensor.gyro - gyro_offset;
    else					 d_theta = 0.0;
    d_theta = d_theta * M_PI / 180.0; // [deg] to [rad]
    // differential distance
    d_x = d_r * cos (d_theta);
    d_y = d_r * sin (d_theta);
    *x = d_x * time * 0.001; // [m]
    *y = d_y * time * 0.001; // [m]
    *theta = -d_theta * time; // [rad]
    //old_ang = d_ang;
}

/*
if(data[i] == 0)			printf("%d=near ", i);
else if(data[i] == 0x1000)	printf("%d=long ", i);
else						printf("%d=%dmm ", i, data[i]);
*/

void RoboCar::GetObstacleData (int *data)
{
    // get obstacle
    const int OBSTACLE_NUM = 8;
    _RcControl.GetObstacleSensorInfoReq (&obstacle);
    for (int i = 0; i < OBSTACLE_NUM; i++)
    {
        data[i] = obstacle.obstacle[i]; // [mm]
    }
}

