/**	RC制御クラスヘッダ.	関口 浩司 	2009-04-20
   Copyright (c) 2009 ZMP Inc. All rights reserved    . **/
/** シミュレーション用に改変　Kosei Demura 2012-5-21**/

#ifdef WINDOWS
#include<Windows.h>
#endif

#include<stdio.h>
#include "RcControl.h"
#include "../main.h"

extern simu::RoboCarClass  *rc;
using namespace zmp::zrc;

/***  RCコントロールクラス.
 * RCをコントロールするためにBaseboardクラスへのアクセスを行う。
 */
RcControl::RcControl()
{

}

RcControl::~RcControl()
{

}

bool RcControl::init()
{

}

bool RcControl::Start()
{

}

bool RcControl::Stop()
{

}

bool RcControl::Close()
{

}


/*** 最大トルクの取得.
 * @param[out]	torque		最大トルク. 1％単位.
 * @retval	true	情報取得成功.
 * @retval	false	情報取得失敗.
 */
bool RcControl::GetMaxTorque(char* torque)
{
    /// 未実装
    return true;
}

/*** 障害物センサ情報の取得要求.
	 * @param[out]	value	障害物センサ情報が設定される構造体アドレス.
	 * @retval	true	情報取得成功.
	 * @retval	false	情報取得失敗.
	 */
bool RcControl::GetObstacleSensorInfoReq(OBSTACLE_VALUE* value)
{
    for (int i=0; i < 8; i++)
    {
        value->obstacle[i] = rc->obstacle[i];
    }
    return true;
}

/*** 電源情報の取得要求
	 * @param[out]	value	電源情報が設定される構造体アドレス.
	 * @retval	true	情報取得成功.
	 * @retval	false	情報取得失敗.
	 */
bool RcControl::GetPowerInfoReq(POWER_VALUE* value)
{
    /// 常にモータへ流れる電流は初期値の1000mA、電圧は7.2Vを返す
    value->motor_current = rc->motor_current;
    value->battery_level   = rc->battery_level;
    return true;
}

/*** サーボの現在位置の取得.
 * @param[out]	angle		present angle. 0.1度単位
 * @retval	true	情報取得成功.
 * @retval	false	情報取得失敗.
 */
bool RcControl::GetPresentAngle(float* angle)
{
    return true;
}

/*** サーボの現在の回転速度取得.
 * @param[out]	speed	present speed. deg/sec単位.
 * @retval	true	情報取得成功.
 * @retval	false	情報取得失敗.
 */
bool RcControl::GetPresentSpeed(int* speed)
{
    return true;
}


/*** センサ情報の取得要求.
 * @param[out]	value	センサ情報が設定される構造体アドレス.
 * @retval	true	情報取得成功.
 * @retval	false	情報取得失敗.
 */
bool RcControl::GetSensorInfoReq(SENSOR_VALUE* value)
{
    value->gyro  =  rc->sensor.gyro;
    value->acc_x = rc->sensor.acc_x;
    value->acc_y = rc->sensor.acc_y;
    value->acc_z = rc->sensor.acc_z;
    value->enc_0 = rc->sensor.enc_0;
    value->enc_1 = rc->sensor.enc_1;
    value->enc_2 = rc->sensor.enc_2;
    value->enc_3 = rc->sensor.enc_3;
    value->enc_4 = rc->sensor.enc_4;

    return true;
}

/*** サーボ情報取得要求
    * BaseBoardに対してServo情報取得メッセージを送信して、現在保持しているServo情報を返す
    * @param[in]		id		サーボID.
    * @param[in]		adr		取得情報アドレス.
    * @param[in]		len		取得情報の長さ.
    * @param[out]	value	サーボ情報が設定される構造体アドレス.
    * @retval	true	情報取得成功.
    * @retval	false	情報取得失敗.
    */
bool RcControl::GetServoInfoReq(int id, SERVO_INFO_ADR adr, int len, DRIVE_VALUE* value)
{
    /// 未実装
    /// 常にモータへ流れる電流は初期値の1000mA、電圧は7.2Vを返す
    value->present_position       =  rc->servo_present_position;
    value->present_current        =  rc->servo_present_current;
    value->present_temperature = rc->servo_present_temperature;
    value->present_volts           = rc->servo_present_volts;
    return true;
}

/** トルク状態の取得.
 * @param[out]	enable		トルク状態. 0=OFF,  1=ON,  2=BRAKE.
 * @retval	true	情報取得成功.
 * @retval	false	情報取得失敗.
 */
bool RcControl::GetTorqueEnable(char* enable)
{
    return true;
}


/*** 駆動速度の設定.
 * @param[in]		speed	駆動速度(mm/sec). 1mm単位で設定範囲は-2800～2800.
 * @retval	true	情報設定成功 * @retval	false	情報設定失敗
 */
bool RcControl::SetDriveSpeed(int speed)
{
    rc->speed = speed;
}

/*** ジャイロオフセット.ジャイロセンサ出力のゼロ点設定.
	 * @param[in]		offset		ジャイロオフセット(deg/s).
	 */
void RcControl::SetGyroOffset(float offset)
{
    /// 未実装
}

/*** 電流オフセット設定.モータ電流のゼロ点設定
 * @param[in]		offset		電流オフセット(A).
 */
void RcControl::SetMotorCurrentOffset(float offset)
{
    /// 未実装
}


/*** 通知情報フラグ設定要求.
* @param[in]	flag		通知する情報を設定するフラグ. bit0=Sensor. bit1=Obstacle. bit2=Power. bit3～bit4=reserve.
  */
void RcControl::SetReportFlagReq(char flag)
{
    //　未実装
}

/*** 操舵角の設定.
 * @param[in]		angle	操舵角. 0.1度単位で設定範囲は-30.0～30.0.
 * @retval	true	情報設定成功
 * @retval	false	情報設定失敗
 */
bool RcControl::SetSteerAngle(float angle)
{
    rc->steer = angle;
}

/*** 最大トルクの設定.
 * @param[in]		torque		最大トルク. 1%単位で設定範囲は0(0%)～100(100%).
 * @retval	true	情報設定成功 @retval	false	情報設定失敗
 */
bool RcControl::SetMaxTorque(int torque)
{
    return true;
}

/*** サーボモータのON/OFF.
 * @param[in]		enable		ON/OFF OFF=0, ON=1, BRAKE=2.
 * @retval	true	情報設定成功
 * @retval	false	情報設定失敗
 */
bool RcControl::SetServoEnable(int enable)
{
    return true;
}

/*** モータON/OFF設定要求.
 * @param[in]	enable		モータON/OFF.
 */
void RcControl::SetMotorEnableReq(char enable)
{

}

/*** モータトルク設定要求.
 * @param[in]		p		P(比例動作).
 * @param[in]		i		I(積分動作).
 * @param[in]		d		D(微分動作).
 * @param[in]		w		width.
 */
void RcControl::SetMotorTorqueReq(char p, char i, char d, char w)
{

}

/* void system(char *text)
{
    // 未実装。文字列を表示するだけ
    printf("%s\n",text);
}

void sleep(int t)
{
    Sleep(t*1000);  // sleep関数はWindowsにないのでSleep関数を使う
} */


