/*** The RoboCar Simulator   2012-05-23 ***************/
/*** Copyright (c) 2012 Kosei Demura  All rights reserved. **/
// localization2.cpp

#include <ode/ode.h>
#include "../common.h"

// 以下のexternはmain.hの中で宣言
extern MyObject base;
extern dJointID joint[NUM];
extern dReal WHEEL_R, WHEEL_W;  // 車輪の半径,幅
extern dReal BASE_LX;                // 台車の半径

/*** ロボットの針路計算 ***/
dReal heading()
{
    const dReal *rot = dBodyGetRotation(base.body);
    dReal r11,r12,r13,r21,r22,r23,r31,r32,r33;
    dReal pitch, yaw, roll;

    r11 = rot[0];
    r12 = rot[1];
    r13 = rot[2];
    r21 = rot[4];
    r22 = rot[5];
    r23 = rot[6];
    r31 = rot[8];
    r32 = rot[9];
    r33 = rot[10];
    // ポインタ的表記では　r11 = *(rot+0); r12 = *(rot+1); r13 = *(rot+2);
    pitch = atan2(-r31, sqrt(r32*r32+r33*r33));
    yaw   = atan2(r21,r11);
    roll  = atan2(r32,r33);
    //printf("roll=%f pitch=%f yaw=%f \n", 180.0*roll/M_PI, 180.0*pitch/M_PI, 180.0*yaw/M_PI);
    return -yaw;  // rotation axis is reverse
}


/*** センサ ***/
// 共分散行列Σの誤差分布を持つセンサデータあるいはカメラによる自己位置をシミュレートする関数
void getPos(double *sensor_pos_x, double *sensor_pos_y, double *sensor_course)
{
    double cov[3];  // 共分散行列

    // 共分散行列に値を入れる
    cov[0] =  SENSOR_COV_X;  // [m]   1行目
    cov[1] =  SENSOR_COV_Y;  // [m]   ２行目
    cov[2] =  SENSOR_COV_THETA;  // [rad] ３行目

    const double *pb = dBodyGetPosition(base.body);

    *sensor_pos_x = pb[0]; // pbは真の自己位置
    *sensor_pos_y = pb[1];
    *sensor_course = heading();

    double r1, r2, r3;
    double m = 0.0;     // 平均

    //ボックス・ミュラー法
    r1 = (double)rand()/RAND_MAX;
    r2 = (double)rand()/RAND_MAX;
    r3 = (double)rand()/RAND_MAX;
    *sensor_pos_x += cov[0]*sqrt(-2*log(r1))*cos(2*M_PI*r2)+m;
    *sensor_pos_y += cov[1]*sqrt(-2*log(r1))*sin(2*M_PI*r2)+m;
    *sensor_course += cov[2]*sqrt(-2*log(r1))*cos(2*M_PI*r3)+m;
}



/** 真の自己位置pos[2]と姿勢(ロボットの進行方向）courseを求める関数　**/
void getRealPos(double pos[2], double *course)
{
    const double *pb = dBodyGetPosition(base.body);

    pos[0] = pb[0]; // pbは真の自己位置
    pos[1] = pb[1];
    *course  = heading();
}


