/*** The RoboCar Simulator   2012-06-27 ***************/
/*** Copyright (c) 2012 Kosei Demura  All rights reserved. **/
// simulation.cpp
// ODEgV~[^̖{̕
// {bg̏pύX\Ȃ悤ɉǒB{fB̈ʒu]sɂύXȂ΂Ȃ
// X = x * cos(theta) - y * sin(theta);
// Y = x * sin(theta)  + y * cos(theta);

#include "../main.h"
#include <time.h>
#ifdef WINDOWS
#include <windows.h>
#endif
#include "simulation.h"


extern simu::RoboCarClass  *rc;
extern void getRealPos(double pos[2], double *course);
extern dReal heading();

float xyz[3] = {START_X, START_Y, 2.0f};
float hpr[3] = { 132.0f, -90.0f, 0.0f};
static int  g_laser_view = 1, g_particle_view = 1;
//float xyz[3] = {  3.86f, 3.80f, 9.86f};
//float hpr[3] = { 117.0f, -24.0f, 0.0f};

/*** [U[̔ˁ@***/
static void emitLaser()
{
    dMatrix3 R1, R2, R3, R4, R5;

    const dReal *p = (const dReal *) dBodyGetPosition(laser.body);
    dReal heading_angle = -heading();
    dReal pitch  = dJointGetHingeAngle(laser_joint);

    dRFromAxisAndAngle(R1,1,0,0, -M_PI/2);
    dRFromAxisAndAngle(R4,1,0,0, pitch);

    for (int i=urg.first_index; i < urg.last_index; i++)
    {
        for (int j=0; j < 3; j++)
        {
            g_contact_pos[i][j] = -1;
        }
    }



    for (int i=urg.first_index; i < urg.last_index; i+=urg.skip_lines)
    {
        dRFromAxisAndAngle(R2,0,0,1,
                           heading_angle +(double) (i - (BEAM_NUM/2))/(BEAM_NUM/2) * LASER_LO_LIMIT * (M_PI/180));
        dMultiply0(R3,R2,R1,3,3,3);
        dMultiply0(R5,R4,R3,3,3,3);
        dGeomSetPosition(beam[i], p[0], p[1], p[2]);
        dGeomSetRotation(beam[i],R5);
    }
}

/*** PSD ̔ˁ@***/
static void emitPSD()
{
    dMatrix3 R1, R2, R3, R4, R5;

    dReal heading_angle = -heading();
    dRFromAxisAndAngle(R1,1,0,0, -M_PI/2);

    for (int i=0; i < PSD_NUM; i++)
    {
        const dReal *p = (const dReal *) dBodyGetPosition(psd[i].body);

        for (int j=0; j < 3; j++)
        {
            g_psd_contact_pos[i][j] = -1;
        }

        dRFromAxisAndAngle(R2,0,0,1,
                           heading_angle + (45.0-45.0 * i)  * (M_PI/180));
        dMultiply0(R3,R2,R1,3,3,3);
        dGeomSetPosition(psd_beam[i], p[0], p[1], p[2]);
        dGeomSetRotation(psd_beam[i],R3);
    }
}


static void rotatePos(dReal x, dReal y, dReal theta, dReal *nx, dReal *ny)
{
    dReal dir= -theta * M_PI/180.0;
    *nx = START_X + x * cos(dir) - y * sin(dir);
    *ny = START_Y + x * sin(dir)  + y * cos(dir);
}



static void makeRoboCar()
{
    dMatrix3 R, R2, R_INIT;
    dMass mass, mass2, mass3, mass4;

    // p. px̎t
    dRFromAxisAndAngle(R_INIT, 0, 0, 1, -START_DIR * M_PI/180.0);

    // 
    base.body  = dBodyCreate(world);
    dMassSetZero(&mass);
    dMassSetBoxTotal(&mass,BASE_M,BASE_LX, BASE_LY, BASE_LZ);
    dBodySetMass(base.body,&mass);
    base.geom = dCreateBox(space,BASE_LX, BASE_LY, BASE_LZ);
    dGeomSetBody(base.geom,base.body);
    dReal x, y;
    rotatePos(0, 0, START_DIR, &x, &y);
    dBodySetPosition(base.body,x,y,START_Z);
    dBodySetRotation(base.body,R_INIT);

    // 
    dReal  neck_posy = 0.1075;
    neck.body  = dBodyCreate(world);
    dMassSetZero(&mass2);
    dMassSetCylinderTotal(&mass2,NECK_M,2,NECK_R,NECK_L);
    dBodySetMass(neck.body,&mass2);
    neck.geom = dCreateCylinder(space,NECK_R,NECK_L);
    dGeomSetBody(neck.geom,neck.body);

    rotatePos(0, neck_posy, START_DIR, &x, &y);
    dBodySetPosition(neck.body,x ,y, START_Z+0.5*NECK_L);
    dBodySetRotation(neck.body,R_INIT);
    dGeomDisable(neck.geom);

    // 2
    dReal  neck2_posy = 0.1075;
    neck2.body  = dBodyCreate(world);
    dMassSetZero(&mass2);
    dMassSetCylinderTotal(&mass2,NECK_M,2,NECK2_R,NECK2_L);
    dBodySetMass(neck2.body,&mass2);

    neck2.geom = dCreateCylinder(space,NECK2_R,NECK2_L);
    dGeomSetBody(neck2.geom,neck2.body);
    rotatePos(0, -neck2_posy-0.01, START_DIR, &x, &y);
    dBodySetPosition(neck2.body,x, y, START_Z+0.5*NECK_L);
    dBodySetRotation(neck2.body,R_INIT);
    dGeomDisable(neck2.geom);

    // ZT LIDAR
    laser.body  = dBodyCreate(world);
    dMassSetZero(&mass3);
    dMassSetCylinderTotal(&mass3,SENSOR_M,2,SENSOR_R, SENSOR_L);
    dBodySetMass(laser.body,&mass3);

    laser.geom = dCreateCylinder(space,SENSOR_R, SENSOR_L);
    dGeomSetBody(laser.geom,laser.body);
    rotatePos(0, neck_posy, START_DIR, &x, &y);
    dBodySetPosition(laser.body,x, y, START_Z+NECK_L+BASE_LZ);
    dBodySetRotation(laser.body,R_INIT);
    dGeomDisable(laser.geom);

    // [Ur[DRayWIgŎD{fB͂ȂD
    for (int i=0; i < BEAM_NUM; i++)
    {
        beam[i] =  dCreateRay(space, 0);
        dGeomRaySetLength(beam[i], BEAM_LENGTH);
    }

    // PSD.ZT
    for (int i=0; i < PSD_NUM; i++)
    {
        psd[i].body = dBodyCreate(world);
        dMassSetZero(&mass4);
        dMassSetBoxTotal(&mass4,PSD_M,PSD_LX, PSD_LY, PSD_LZ);
        dBodySetMass(psd[i].body,&mass4);

        dReal x =  0.08 * cos((135-45*i)*M_PI/180.0);
        dReal y =  0.08 * sin((135-45*i)*M_PI/180.0);
        dReal z =  START_Z+BASE_LZ+0.03;
        if (i < 3)  y  += 0.13;
        else       y -=  0.12;

        dReal x1, y1;
        rotatePos(x, y, START_DIR, &x1, &y1);
        dBodySetPosition(psd[i].body, x1, y1,z);

        //dRFromAxisAndAngle(R2, 0, 0, 1, (135-45 * i) * M_PI/180);
        dRFromAxisAndAngle(R2, 0, 0, 1, ((135-45 * i) - START_DIR)* M_PI/180);
        dBodySetRotation(psd[i].body,R2);

        psd_beam[i] =  dCreateRay(space, 0);
        dGeomRaySetLength(psd_beam[i], PSD_LENGTH);
        //dGeomRaySet(psd_beam[i], x, y ,z, -cos(45*i*M_PI/180), -sin(45*i*M_PI/180),0 );
        //dGeomRaySet(psd_beam[i], x, y ,z, -cos((45*i)*M_PI/180), -sin((45*i)*M_PI/180),0 );
        //dGeomRaySet(psd_beam[i], x, y ,z, 1, 1,0 );
    }

    // ԗ
    dRFromAxisAndAngle(R,sin(START_DIR*M_PI/180.0), cos(START_DIR*M_PI/180.0),0,M_PI/2.0);

    for (int i = 0; i < NUM; i++)
    {
        wheel[i].body = dBodyCreate(world);

        dMassSetZero(&mass);
        dMassSetCylinderTotal(&mass,WHEEL_M, 2, WHEEL_R, WHEEL_W);
        dBodySetMass(wheel[i].body,&mass);
        wheel[i].geom = dCreateCylinder(space, WHEEL_R, WHEEL_W);
        dGeomSetBody(wheel[i].geom, wheel[i].body);
        dBodySetRotation(wheel[i].body,R);
    }

    dReal wx = 0.5 * (BASE_LX);
    dReal wy = 0.128;
    dReal wz = START_Z;

    dReal x2, y2;
    rotatePos(wx, wy, START_DIR, &x2, &y2);
    dBodySetPosition(wheel[FRONT_RIGHT].body, x2, y2, START_Z);

    rotatePos(-wx, +wy, START_DIR, &x2, &y2);
    dBodySetPosition(wheel[FRONT_LEFT].body,   x2, y2, START_Z);

    rotatePos(+wx, -wy, START_DIR, &x2, &y2);
    dBodySetPosition(wheel[REAR_RIGHT].body,   x2, y2, START_Z);

    rotatePos(-wx, -wy, START_DIR, &x2, &y2);
    dBodySetPosition(wheel[REAR_LEFT].body,   x2, y2, START_Z);

    /****  qWWCg ****/
    // ǗցiOցj
    dReal ax = cos(START_DIR*M_PI/180.0);
    dReal ay = sin(START_DIR*M_PI/180.0);

    joint[FRONT_RIGHT] = dJointCreateHinge2(world,0);
    dJointAttach(joint[FRONT_RIGHT], base.body, wheel[FRONT_RIGHT].body);
    const dReal *a = dBodyGetPosition(wheel[FRONT_RIGHT].body);
    dJointSetHinge2Anchor(joint[FRONT_RIGHT], a[0], a[1], a[2]);
    //dJointSetHinge2Anchor(joint[FRONT_RIGHT], START_X+wx, START_Y+wy,  wz-WHEEL_R);
    dJointSetHinge2Axis1(joint[FRONT_RIGHT],0, 0, 1);
    dJointSetHinge2Axis2(joint[FRONT_RIGHT], ax, ay, 0);
    dJointSetHinge2Param (joint[FRONT_RIGHT], dParamLoStop, -30*M_PI/180.0);
    dJointSetHinge2Param (joint[FRONT_RIGHT], dParamHiStop,   30*M_PI/180.0);
    //dJointSetHinge2Param (joint[FRONT_RIGHT],dParamSuspensionERP,0.9);
    //dJointSetHinge2Param (joint[FRONT_RIGHT],dParamSuspensionCFM,0.01);

    joint[FRONT_LEFT] = dJointCreateHinge2(world,0);
    dJointAttach(joint[FRONT_LEFT], base.body, wheel[FRONT_LEFT].body);
    const dReal *b = dBodyGetPosition(wheel[FRONT_LEFT].body);
    dJointSetHinge2Anchor(joint[FRONT_LEFT], b[0], b[1], b[2]);
    //dJointSetHinge2Anchor(joint[FRONT_LEFT], START_X-wx,  START_Y+wy, wz-WHEEL_R);
    dJointSetHinge2Axis1(joint[FRONT_LEFT],0, 0, 1);
    dJointSetHinge2Axis2(joint[FRONT_LEFT],ax, ay, 0);
    dJointSetHinge2Param (joint[FRONT_LEFT], dParamLoStop, -30*M_PI/180.0);
    dJointSetHinge2Param (joint[FRONT_LEFT], dParamHiStop,   30*M_PI/180.0);
    //dJointSetHinge2Param (joint[FRONT_LEFT],dParamSuspensionERP,0.9);
    //dJointSetHinge2Param (joint[FRONT_LEFT],dParamSuspensionCFM,0.01);


    // 쓮ցiցj
    joint[REAR_RIGHT] = dJointCreateHinge(world,0);
    dJointAttach(joint[REAR_RIGHT], base.body, wheel[REAR_RIGHT].body);
    const dReal *c = dBodyGetPosition(wheel[REAR_RIGHT].body);
    dJointSetHingeAnchor(joint[REAR_RIGHT], c[0], c[1], c[2]);
    dJointSetHingeAxis(joint[REAR_RIGHT],ax, ay, 0);


    joint[REAR_LEFT] = dJointCreateHinge(world,0);
    dJointAttach(joint[REAR_LEFT], base.body, wheel[REAR_LEFT].body);
    const dReal *d = dBodyGetPosition(wheel[REAR_LEFT].body);
    dJointSetHingeAnchor(joint[REAR_LEFT], d[0], d[1], d[2]);
    dJointSetHingeAxis(joint[REAR_LEFT], ax, ay, 0);

    // neck joints
    neck_joint = dJointCreateHinge(world,0);
    dJointAttach(neck_joint,neck.body,base.body);
    dJointSetHingeAxis(neck_joint,0, 0, 1);
    //dJointSetHingeAnchor(neck_joint, START_X, START_Y+0.5*BASE_S[1]-NECK_R,START_Z);
    rotatePos(0, 0, START_DIR, &x, &y);
    dJointSetHingeAnchor(neck_joint, x, y, START_Z);
    dJointSetHingeParam(neck_joint,dParamLoStop, 0);
    dJointSetHingeParam(neck_joint,dParamHiStop, 0);


    // neck2 joints
    neck2_joint = dJointCreateHinge(world,0);
    dJointAttach(neck2_joint,neck2.body,base.body);
    dJointSetHingeAxis(neck2_joint,0, 0, 1);
    //dJointSetHingeAnchor(neck_joint, START_X, START_Y+0.5*BASE_S[1]-NECK_R,START_Z);
    rotatePos(0, 0, START_DIR, &x, &y);
    dJointSetHingeAnchor(neck2_joint, x, y,START_Z);
    dJointSetHingeParam(neck2_joint,dParamLoStop, 0);
    dJointSetHingeParam(neck2_joint,dParamHiStop, 0);

    // sensor joints
    laser_joint = dJointCreateHinge(world,0);
    dJointAttach(laser_joint,neck.body,laser.body);
    dJointSetHingeAxis(laser_joint, ax, ay,0);
    //dJointSetHingeAnchor(sensor_joint,START_X, START_Y+0.5*BASE_S[1]-0.1*SENSOR_S[1],START_Z+NECK_L);
    rotatePos(0, 0, START_DIR, &x, &y);
    dJointSetHingeAnchor(laser_joint, x, y,START_Z+NECK_L);

    dJointSetHingeParam(laser_joint,dParamLoStop, 0.0 * M_PI);
    dJointSetHingeParam(laser_joint,dParamHiStop, 0.0 * M_PI);

    // psd joints
    for (int i= 0; i < PSD_NUM; i++)
    {
        psd_joint[i] = dJointCreateFixed(world,0);
        dJointAttach(psd_joint[i],psd[i].body,base.body);
        dJointSetFixed(psd_joint[i]);
    }
}

#define CORRIDOR  (WALL_LENGTH_X - WALL2_LENGTH_X)/2.0
#define OFFSET_XY -(WALL_LENGTH_X - WALL2_LENGTH_X)/4.0
/*** ǂ̐ ***/
static void makeWall()
{
    dReal pos[WALL_PARTS_NUM][3] =
    {
        //1
        {X_OFFSET+WALL_SIDES[0][0]/2.0, WALL_SIDES[0][1]/2.0-Y_OFFSET,  1.0},
        {-X_OFFSET+WALL_SIDES[1][0]/2.0, -WALL_SIDES[1][1]/2.0-Y_OFFSET,  1.0},
        {X_OFFSET+WALL_SIDES[0][0]+WALL_SIDES[2][0]/2.0, WALL_SIDES[2][1]/2.0+WALL_SIDES[0][1]-Y_OFFSET,  1.0},
        {X_OFFSET+WALL_SIDES[3][0]/2.0, WALL_SIDES[3][1]/2.0+WALL_SIDES[2][1]+WALL_SIDES[0][1]-Y_OFFSET,  1.0},
        {X_OFFSET+WALL_SIDES[3][0]+WALL_SIDES[4][0]/2.0, WALL_SIDES[4][1]/2.0+WALL_SIDES[3][1]+WALL_SIDES[2][1]+WALL_SIDES[0][1]-Y_OFFSET, 1.0},
        {-X_OFFSET+WALL_SIDES[5][0]/2.0, WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[5][1]/2.0-Y_OFFSET, 1.0},

        //2
        {-X_OFFSET-WALL_SIDES[6][0]/2.0, WALL_SIDES[6][1]/2.0-Y_OFFSET,  1.0},
        {-X_OFFSET-WALL_SIDES[7][0]/2.0,WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]/2.0-Y_OFFSET-0.72, 1.0 },
        {-X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]/2.0,WALL_SIDES[6][1]+2.35+WALL_SIDES[8][1]/2.0-Y_OFFSET,1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-2.29-WALL_SIDES[9][0]/2.0, WALL_SIDES[6][1]+2.35-WALL_SIDES[9][1]/2.0, 1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-6.77-WALL_SIDES[9][0]-WALL_SIDES[10][0]/2.0, WALL_SIDES[6][1]+2.35-WALL_SIDES[10][1]/2.0-Y_OFFSET, 1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-14.27-WALL_SIDES[11][0]-WALL_SIDES[11][0]/2.0, WALL_SIDES[6][1]+2.35-WALL_SIDES[11][1]/2.0-Y_OFFSET, 1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-5.80-WALL_SIDES[12][0]/2.0,WALL_SIDES[6][1]+2.35-WALL_SIDES[12][1]/2.0-Y_OFFSET, 1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-WALL_SIDES[13][0]/2.0,WALL_SIDES[6][1]+2.35+WALL_SIDES[13][1]/2.0-Y_OFFSET, 1.0},

        //3
        {-X_OFFSET-WALL_SIDES[6][0]-1.8-WALL_SIDES[14][0]/2.0,WALL_SIDES[6][1]+2.35-WALL_SIDES[14][1]/2.0-Y_OFFSET, 1.0},
        {-X_OFFSET-WALL_SIDES[6][0]+WALL_SIDES[15][0]/2.0,-Y_OFFSET-WALL_SIDES[15][1]/2.0,1.0},
        {-X_OFFSET-WALL_SIDES[6][0]-1.8-WALL_SIDES[16][0]/2.0,-Y_OFFSET-WALL_SIDES[14][1]+3.0+WALL_SIDES[16][1]/2.0,1.0},

        //4
        {-X_OFFSET-4.92-1.08-WALL_SIDES[17][0]/2.0, -Y_OFFSET+1.50+WALL_SIDES[6][1]+2.35+WALL_SIDES[17][1]/2.0,1.0},
        {-X_OFFSET-4.92-1.08-1.70-WALL_SIDES[17][0]-WALL_SIDES[18][0]/2.0, -Y_OFFSET+1.50+WALL_SIDES[6][1]+2.35+WALL_SIDES[18][1]/2.0,1.0},
        {-X_OFFSET+WALL_SIDES[19][0]/2.0,WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]-WALL_SIDES[19][1]/2.0-Y_OFFSET-0.72, 1.0},
        {-X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]/2.0,
         WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]-WALL_SIDES[19][1]-WALL_SIDES[20][1]/2.0-Y_OFFSET-0.72,
         1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[21][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]-WALL_SIDES[21][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        //5
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[22][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[23][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[23][0]-(WALL_SIDES[22][0]-WALL_SIDES[21][0])+WALL_SIDES[24][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]+WALL_SIDES[24][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[25][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]+WALL_SIDES[24][1]+WALL_SIDES[25][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[23][0]-(WALL_SIDES[22][0]-WALL_SIDES[21][0])+WALL_SIDES[24][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]+WALL_SIDES[24][1]+WALL_SIDES[25][1]+WALL_SIDES[26][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[25][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]+WALL_SIDES[24][1]+WALL_SIDES[25][1]+WALL_SIDES[26][1]+WALL_SIDES[27][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]-WALL_SIDES[28][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[23][1]+WALL_SIDES[24][1]+WALL_SIDES[25][1]+WALL_SIDES[26][1]+WALL_SIDES[27][1]+WALL_SIDES[28][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]-WALL_SIDES[28][0]-WALL_SIDES[29][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[29][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]-WALL_SIDES[28][0]-WALL_SIDES[29][0]-WALL_SIDES[30][0]/2.0-(WALL_SIDES[22][0]-WALL_SIDES[21][0]),
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        //6
        {-X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55-WALL_SIDES[31][0]/2.0,WALL_SIDES[6][1]+2.35+WALL_SIDES[31][1]/2.0-Y_OFFSET, 1.0},
        {-X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]/2.0+1.9,
         WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]/2.0-Y_OFFSET-0.72-0.1,
         1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]+WALL_SIDES[33][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]+WALL_SIDES[33][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]+WALL_SIDES[36][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]+WALL_SIDES[33][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]+WALL_SIDES[36][1]+WALL_SIDES[37][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]-WALL_SIDES[36][0]+WALL_SIDES[38][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]+WALL_SIDES[36][1]+WALL_SIDES[37][1]+WALL_SIDES[38][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        //7
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55-WALL_SIDES[39][0]/2.0,
            WALL_SIDES[6][1]+2.35+WALL_SIDES[31][1]+WALL_SIDES[39][1]/2.0-Y_OFFSET-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55-WALL_SIDES[39][0]-WALL_SIDES[40][0]/2.0,
            WALL_SIDES[6][1]+2.35+WALL_SIDES[31][1]+WALL_SIDES[39][1]+WALL_SIDES[40][1]/2.0-Y_OFFSET-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55-WALL_SIDES[39][0]-WALL_SIDES[40][0]+WALL_SIDES[41][0]/2.0,
            WALL_SIDES[6][1]+2.35+WALL_SIDES[31][1]+WALL_SIDES[39][1]+WALL_SIDES[40][1]+WALL_SIDES[41][1]/2.0-Y_OFFSET-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]-WALL_SIDES[36][0]+WALL_SIDES[38][0]+WALL_SIDES[42][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]+WALL_SIDES[36][1]+WALL_SIDES[37][1]+WALL_SIDES[38][1]+WALL_SIDES[42][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET-WALL_SIDES[7][0]-0.96-WALL_SIDES[8][0]-2.1-2.55+WALL_SIDES[32][0]-WALL_SIDES[36][0]+WALL_SIDES[38][0]+WALL_SIDES[42][0]-WALL_SIDES[43][0]/2.0+1.9,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[22][1]+WALL_SIDES[32][1]+WALL_SIDES[33][1]+WALL_SIDES[34][1]+WALL_SIDES[35][1]+WALL_SIDES[36][1]+WALL_SIDES[37][1]+WALL_SIDES[38][1]+WALL_SIDES[42][1]-WALL_SIDES[43][1]/2.0-Y_OFFSET-0.72-0.1,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[21][0]+WALL_SIDES[44][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+WALL_SIDES[44][1]/2.0-Y_OFFSET-0.72-0.55,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[21][0]-WALL_SIDES[45][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[45][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[21][0]-WALL_SIDES[45][0]+0.1+WALL_SIDES[46][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[45][1]+WALL_SIDES[46][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
        {
            -X_OFFSET+WALL_SIDES[19][0]+WALL_SIDES[20][0]+WALL_SIDES[21][0]-WALL_SIDES[45][0]+0.1+WALL_SIDES[46][0]+WALL_SIDES[47][0]/2.0,
            WALL_SIDES[0][1]+WALL_SIDES[2][1]+WALL_SIDES[3][1]+WALL_SIDES[4][1]+WALL_SIDES[7][1]+1.88+WALL_SIDES[45][1]+WALL_SIDES[46][1]-WALL_SIDES[47][1]/2.0-Y_OFFSET-0.72,
            1.0
        },
    };

    for (int i = 0; i < WALL_PARTS_NUM ; i++)
    {
        if (i != 26)
        {
            wall_parts[i] = dCreateBox(space, WALL_SIDES[i][0],
                                       WALL_SIDES[i][1], WALL_SIDES[i][2]);
            dGeomSetPosition(wall_parts[i],pos[i][0], pos[i][1], pos[i][2]);
        }
        else
        {
            wall_parts[i] = dCreateBox(space, WALL_SIDES[i][0],
                                       WALL_SIDES[i][1], WALL_SIDES[i][2]);
            dGeomSetPosition(wall_parts[i],pos[i][0]+ 1-0.42, pos[i][1]+0.81, pos[i][2]);
            // while (1) printf("goal pos=%f %f",(dReal) pos[i][0]+1, (dReal) pos[i][1]+0.8);
        }
    }
}


/*** ǂ̕` ***/
static void drawWall()
{
    dsSetTexture(DS_NONE);
    for (int i = 0; i < WALL_PARTS_NUM; i++)
    {
        if (i != 26) dsSetColor(1.3, 1.3, 1.3);
        else        dsSetColor(1.3, 0.0, 0.0);

        if (i != 26)
        {
            dsDrawBox(dGeomGetPosition(wall_parts[i]),
                      dGeomGetRotation(wall_parts[i]),WALL_SIDES[i]);
        }
    }

    // Goaliԁj̍W (2, 25.5)
    dsSetColor(1.3,0.0, 0.0);
    dsDrawCylinder((const dReal *) dGeomGetPosition(wall_parts[26]),
                   dGeomGetRotation(wall_parts[26]),5.0, 0.1);
    dReal *tmp = (dReal *) dGeomGetPosition(wall_parts[26]);
    //while (1) printf("Goal=%f %f",tmp[0],tmp[1]);
}

/*** _̕` ***/
static void drawPole()
{
    // Startij̍W (-2, 6)
    dsSetColor(0.0, 0.0, 1.3);
    //dReal pos[3] = {-2,6,0.1};
    dReal pos[3] = {0,0.1};
    dsDrawCylinder((const dReal *) pos,
                   dGeomGetRotation(wall_parts[26]),0.2, 0.1);
}

/*** ~̕` ***/
static void drawCircle(dReal r,dReal center[2], int angle1, int angle2)
{
    dReal pos1[3],pos2[3],z=0.005;
    pos1[0] = r * cos(angle1 * M_PI/180.0) + center[0];
    pos1[1] = r * sin(angle1 * M_PI/180.0) + center[1];
    pos1[2] = z;

    for (int i = angle1; i < angle2; i++)
    {
        pos2[0] = r* cos(i * M_PI/180.0) + center[0];
        pos2[1] = r* sin(i * M_PI/180.0) + center[1];
        pos2[2] = z;
        dsDrawLine(pos1,pos2);
        pos1[0] = pos2[0];
        pos1[1] = pos2[1];
        pos1[2] = pos2[2];
    }
}

/*** ̕` ***/
static void drawCourse()
{
    dReal z = 0.005;
    dReal center_r = 1.0, corner_r = 0.5;
    dReal lx = 10, ly = 10;
    dReal pos[][3] = {{lx,ly,z},{-lx,ly,z},{-lx,-ly,z},{lx,-ly,z}};
    dsSetColor(1.3,1.3,1.3);
    dsDrawLine(pos[0],pos[1]);
    dsDrawLine(pos[1],pos[2]);
    dsDrawLine(pos[2],pos[3]);
    dsDrawLine(pos[3],pos[0]);

    // X^[gT[N̕`
    dReal pos_center[2] = {50,50};
    drawCircle(center_r,pos_center, 0, 360);

}

/*** X^[g֐ ***/
static void start()
{
    dsSetViewpoint(xyz,hpr);
    dsSetSphereQuality(3);
}

/*** R[obN֐ ***/
static void nearCallback (void *data, dGeomID o1, dGeomID o2)
{
    int i,n;

    dBodyID b1 = dGeomGetBody(o1);
    dBodyID b2 = dGeomGetBody(o2);
    if (b1 && b2 && dAreConnectedExcluding(b1,b2,dJointTypeContact)) return;

    static const int N = 10;
    dContact contact[N];
    n = dCollide(o1,o2,N,&contact[0].geom,sizeof(dContact));
    if (n > 0)
    {
        for (i=0; i<n; i++)
        {

            if ((o1 == wheel[FRONT_LEFT].geom) || (o2 == wheel[FRONT_RIGHT].geom) ||
                    (o1 == wheel[REAR_LEFT].geom)  || (o2 == wheel[REAR_RIGHT].geom))
            {
                contact[i].surface.mode = dContactSoftERP | dContactSoftCFM | dContactApprox1;
                //contact[i].surface.mode = dContactSoftERP | dContactSoftCFM ;

                contact[i].surface.mu = 1.0; // dInfinity; // CW0
                //contact[i].surface.slip1    = 1.0;      // PC̊
                //contact[i].surface.slip2    = 1.0;      // QC̊
                contact[i].surface.soft_erp = 0.9;       // nʂERP
                contact[i].surface.soft_cfm = 1e-4;      // nʂCFM
            }
            else
            {
                // [Ur[̏Փ˔. NXB
                if (dGeomGetClass(o1) == dRayClass || dGeomGetClass(o2) == dRayClass)
                {
                    for (int k=0; k < BEAM_NUM; k+= urg.skip_lines)
                    {
                        if (o1 == beam[k] || o2 == beam[k])
                        {
                            // ̂ȂȂ߂
                            if ((g_contact_pos[k][0] == -1) && (g_contact_pos[k][1] == -1)
                                    && (g_contact_pos[k][2]==-1))
                            {
                                for (int j=0; j < 3; j++)
                                    g_contact_pos[k][j] = contact[i].geom.pos[j];
                            }
                            else
                            {
                                //oOC@N
                                double pos[3], th;
                                dmGetRealPosition(pos,&th);

                                double dist1 = sqrt(pow(g_contact_pos[k][0]- pos[0],2)
                                                    + pow(g_contact_pos[k][1]- pos[1],2));
                                double dist2 = sqrt(pow(contact[i].geom.pos[0]- pos[0],2)
                                                    + pow(contact[i].geom.pos[1]- pos[1],2));
                                if (dist1 > dist2)
                                {
                                    for (int j=0; j < 3; j++) g_contact_pos[k][j] = contact[i].geom.pos[j];
                                }

                                return;
                            }
                        }
                    }
                    for (int k2=0; k2 < PSD_NUM; k2++)
                    {
                        if (o1 == psd_beam[k2] || o2 == psd_beam[k2])
                        {
                            // ̂ȂȂ߂
                            if ((g_psd_contact_pos[k2][0] == -1) && (g_psd_contact_pos[k2][1] == -1)
                                    && (g_psd_contact_pos[k2][2]==-1))
                            {
                                for (int j=0; j < 3; j++)
                                    g_psd_contact_pos[k2][j] = contact[i].geom.pos[j];
                            }
                            else
                            {
                                //oOC@N
                                double pos[3], th;
                                dmGetRealPosition(pos,&th);

                                double dist1 = sqrt(pow(g_psd_contact_pos[k2][0]- pos[0],2)
                                                    + pow(g_psd_contact_pos[k2][1]- pos[1],2));
                                double dist2 = sqrt(pow(contact[i].geom.pos[0]- pos[0],2)
                                                    + pow(contact[i].geom.pos[1]- pos[1],2));
                                if (dist1 > dist2)
                                {
                                    for (int j=0; j < 3; j++)
                                        g_psd_contact_pos[k2][j] = contact[i].geom.pos[j];
                                }

                                return;
                            }
                        }
                    }
                    return; // ڐG_邾Ȃ̂Ŗ߂
                }
                contact[i].surface.mode = // dContactSlip1 | dContactSlip2 |
                    dContactSoftERP | dContactSoftCFM | dContactApprox1 | dContactMu2;
                //contact[i].surface.mode = dContactSoftERP | dContactSoftCFM | dContactMu2;

                contact[i].surface.mu   = 1.0; // dInfinity;//1.0; // dInfinity; // CW
                contact[i].surface.mu2 = 1.0;//0.1; // CW

                contact[i].surface.slip1    = 0.0001;      // PC̊
                contact[i].surface.slip2    = 0.0001;      // QC̊
                contact[i].surface.soft_erp = 0.9;       // nʂERP
                contact[i].surface.soft_cfm = 1e-4;      // nʂCFM
            }

            dJointID c = dJointCreateContact(world,contactgroup,&contact[i]); //ڐGWCg̐
            dJointAttach(c,b1,b2);                   // WCǧ
        }
    }
}

/*** L[{[hR}h̏֐ ***/
static void command(int cmd)
{

    switch (cmd)
    {
    case '0':
        float xyz1[3],hpr1[3];
        dsGetViewpoint(xyz1,hpr1);
        printf("xyz=%4.2f %4.2f %4.2f ",xyz1[0],xyz1[1],xyz1[2]);
        printf("hpr=%6.2f %6.2f %5.2f \n",hpr1[0],hpr1[1],hpr1[2]);
        //while (1);
        break;
        //case 'j': v_r += 0.1; break;     // Eԗ,Oi
        //case 'f': v_l += 0.1; break;     // ԗ,Oi
        //case 'k': v_r -= 0.1; break;     // Eԗ,i
        //case 'd': v_l -= 0.1; break;     // ԗ,i
    case '1':
        xyz[0] = -10.42f;
        xyz[1] =   15.54f;
        xyz[2] =  26.0f;
        hpr[0] =   90.0f;
        hpr[1] = -90.0f;
        hpr[2] =   0.0f;
        dsSetViewpoint(xyz,hpr);
        break;
    case '2':
    {
        LAST_KEY = cmd;
        VIEW_MODE = !VIEW_MODE;
        const dReal *pos = (const dReal *) dBodyGetPosition(base.body);
        xyz[0] = pos[0];
        xyz[1] = pos[1];
        xyz[2] = 4.0;
        hpr[0] = - (heading() * 180.0/M_PI - 90.0);
        hpr[1] = -90;
        hpr[2] = 0;
        dsSetViewpoint(xyz,hpr);
        //printf("Current Pos x=%f  y=%f\n",xyz[0],xyz[1]);
        break;
    }
    case '3':
    {
        LAST_KEY = cmd;
        VIEW_MODE = !VIEW_MODE;
        const dReal *pos = (const dReal *) dBodyGetPosition(base.body);
        xyz[0] = pos[0];
        xyz[1] = pos[1];
        xyz[2] = 1.0;
        hpr[0] = - (heading() * 180.0/M_PI - 90.0);
        hpr[1] = 0;
        hpr[2] = 0;
        dsSetViewpoint(xyz,hpr);
        printf("Current Pos x=%f  y=%f\n",xyz[0],xyz[1]);
        break;
    }
    case '4':
        xyz[0] =   3.86f;
        xyz[1] =   3.80f;
        xyz[2] =   9.86f;
        hpr[0] = 132.0f;
        hpr[1] = -40.0f;
        hpr[2] =   0.0f;
        dsSetViewpoint(xyz,hpr);
        break;
    case '5':
        xyz[0] =  -3.38f;
        xyz[1] =   1.83f;
        xyz[2] =   24.0f;
        hpr[0] = 113.0f;
        hpr[1] = -63.0f;
        hpr[2] =   0.0f;
        dsSetViewpoint(xyz,hpr);
        break;
    case 's':
        rc->speed -= 50; // [mm]
        break;     // i
    case 'w':
        rc->speed += 50;
        break;     // Oi
    case 'd':
        rc->steer += 1;
        break;     // E]
    case 'a':
        rc->steer -= 1;
        break;     // ]
    case ' ':
        rc->speed = 0.0;
        break; //   ~
    case 'l':
        if (g_laser_view == 0)  g_laser_view = 1;
        else                        g_laser_view = 0;
        break;
    case 'p':
        if (g_particle_view == 0)  g_particle_view = 1;
        else                           g_particle_view = 0;
        break;
    case 'r':
        initSimu();
        break;

    default:
        printf("\nInput 1,2,3,4,a,d,l,s,w:%d \n",cmd);
        break;
    }
}


// tB[hWn烍{bgWnւ̕ϊ  vO4.2
static void field2RobotPos(const dReal p[2], const dReal c[2], dReal theta,
                           dReal pr[2], dReal *r, dReal *phi)
{
    pr[0] =   (p[0]-c[0]) * cos(theta) + (p[1]-c[1]) * sin(theta);
    pr[1] = - (p[0]-c[0]) * sin(theta) + (p[1]-c[1]) * cos(theta);

    *r   = sqrt(pr[0] * pr[0] + pr[1] * pr[1]);  // ̂܂ł̋
    *phi = atan2(pr[0],pr[1]);                   // ̂̊px
}

static void drawRoboCar()
{
    // 
    dsSetColor(1.3,1.3,1.3);
    dsDrawBox(dGeomGetPosition(base.geom),
              dGeomGetRotation(base.geom),BASE_SIZE);

    // 
    //dsSetColor(1.0,1.0,1.0);
    dsDrawCylinder(dBodyGetPosition(neck.body),
                   dBodyGetRotation(neck.body),NECK_L,NECK_R);

    // 2
    //dsSetColor(1.0,1.0,1.0);
    dsDrawCylinder(dBodyGetPosition(neck2.body),
                   dBodyGetRotation(neck2.body),NECK2_L,NECK2_R);



    // LIDAR
    dsSetColorAlpha(0.0,0.0,0.0,1.0);
    dsDrawCylinder(dGeomGetPosition(laser.geom),
                   dGeomGetRotation(laser.geom),SENSOR_L, SENSOR_R);

    // PSD
    for (int i = 0; i < PSD_NUM; i++)
    {
        if (i==0)  dsSetColor(1.0, 0.0, 0.0);
        else       dsSetColor(0.0, 0.0, 0.0);
        dsDrawBox(dBodyGetPosition(psd[i].body),
                  dBodyGetRotation(psd[i].body),PSD_SIZE);
    }

    // ԗ
    dsSetColor(0.0,0.0,0.0);
    static dReal wheel_width = 0.02;
    for (int i=0; i< NUM; i++)
    {
        dsDrawCylinder(dBodyGetPosition(wheel[i].body),
                       dBodyGetRotation(wheel[i].body),wheel_width, WHEEL_R);
    }
}

//KEXzɏ]֐
double gaussrand(double *x, double *y, double *theta)
{
    double cov[3];  // Us
    // Usɒl
    cov[0] =  SENSOR_COV_X;  // [m]   1s
    cov[1] =  SENSOR_COV_Y;  // [m]   Qs
    cov[2] =  SENSOR_COV_THETA;  // [rad] Rs
    double r1=0.0, r2=0.0, r3=0.0;
    double m = 0.0;     // 

    //{bNXE~[@
    r1 = (double)rand()/RAND_MAX;
    r2 = (double)rand()/RAND_MAX;
    r3 = (double)rand()/RAND_MAX;
    *x = cov[0]*sqrt(-2*log(r1))*cos(2*M_PI*r2)+m;
    *y = cov[1]*sqrt(-2*log(r1))*sin(2*M_PI*r2)+m;
    *theta = cov[2]*sqrt(-2*log(r1))*cos(2*M_PI*r3)+m;
}

/*** [U̕` ***/
void drawLaser()
{
    const dReal *pos;
    dsSetColorAlpha(1.0, 0.0, 0.0, 0.5);
    static int skip = 6; // `ɂ邽߂̊Ԉ

    pos = dBodyGetPosition(laser.body);
    for (int i = 0; i < BEAM_NUM; i++)
    {
        if (!((g_contact_pos[i][0] == -1) && (g_contact_pos[i][1] == -1) &&
                (g_contact_pos[i][2] == -1)))
        {
            // `ɂ邽߂ɊԈ
            if ((i % skip) == 0)dsDrawLine(pos,(const double *) g_contact_pos[i]);
        }
    }
}

/*** PSD̕` ***/
void drawPSD()
{
    dsSetColorAlpha(0.0, 0.0, 1.0, 0.5);

    for (int i = 0; i < PSD_NUM; i++)
    {
        const dReal *pos = dBodyGetPosition(psd[i].body);
        if (!((g_psd_contact_pos[i][0] == -1) && (g_psd_contact_pos[i][1] == -1) &&
                (g_psd_contact_pos[i][2] == -1)))
        {
            dsDrawLine(pos,(const double *) g_psd_contact_pos[i]);
        }
    }
}

void drawPSD2()
{
    dVector3 a, dir;

    dsSetColorAlpha(0.0, 0.0, 1.0, 0.5);
    for (int i=0; i< PSD_NUM; i++)
    {
        const dReal *pos = dBodyGetPosition(psd[i].body);
        //dGeomRayGet(psd_beam[i], a, dir);
        //dReal length = dGeomRayGetLength(psd_beam[i]);

        dVector3 b;
        //b[0] = a[0] + (dir[0] * length);
        //b[1] = a[1] + (dir[1] * length);
        //b[2] = a[2] + (dir[2] * length);
        b[0] = g_psd_contact_pos[i][0];
        b[1] = g_psd_contact_pos[i][1];
        b[2] = g_psd_contact_pos[i][2];

        int data;
        if (!((b[0]==-1)&&(b[1]==-1)&&(b[2]==-1)))
        {
            dsDrawLine(pos, b);
            /* dReal d2 = 0;
            for (int j = 0; j < 3; j++)
            {
                d2 += (pos[j]-g_contact_pos[i][j]) * (pos[j] - g_contact_pos[i][j]);
            }
            data = (long) (1000 * sqrt(d2));
            printf("PSD2 dist[%d]=%d[mm] \n",i,data);
            printf("P=%.1f %.1f %.1",pos[0], pos[1],pos[2]);
            printf("C=%.1f %.1f %.1",g_pos[0], pos[1],pos[2]); */
        }

    }
}

void drawPSD3()
{
    dVector3 a, dir;

    dsSetColorAlpha(0.0, 0.0, 1.0, 0.5);
    for (int i=0; i< PSD_NUM; i++)
    {
        const dReal *pos = dBodyGetPosition(psd[i].body);
        dGeomRayGet(psd_beam[i], a, dir);
        //dReal length = dGeomRayGetLength(psd_beam[i]);

        dVector3 b;
        //b[0] = a[0] + (dir[0] * length);
        //b[1] = a[1] + (dir[1] * length);
        //b[2] = a[2] + (dir[2] * length);
        b[0] = g_psd_contact_pos[i][0];
        b[1] = g_psd_contact_pos[i][1];
        b[2] = g_psd_contact_pos[i][2];
        if (!((b[0]==-1)&&(b[1]==-1)&&(b[2]==-1)))
        {
            dsDrawLine(a, b);
        }

    }
}

void drawSphere2()
{
    dReal pos[3] = {0, 0, 1};
    dMatrix3 R;
    dRSetIdentity(R);
    dsSetColor (0, 0, 1);
    dsDrawSphere(pos, R, 0.1);
}



/*** p[eBN̕` ***/
static void drawParticles()
{
    static int counter = 0;
    //static dReal pos[PARTICLE_NUM][3];
    //static dReal theta[PARTICLE_NUM][2];


    dMatrix3 R;          // ]s
    dReal rb = 0.01;      //0.01; // p[eBN̔a
    dsSetColor(1,0,0); //  0 ~1
    dRSetIdentity(R); // ]sPʍsɐݒ

    /* double p_x=0, p_y=0, p_theta=0;


    for (int i=0; i < PARTICLE_NUM; i++)
    {
        gaussrand(&p_x,&p_y,&p_theta);
        pos[i][0] = START_X + p_x;
        pos[i][1] = START_Y + p_y;
        pos[i][2] = 0.02;
        theta[i][0] = START_DIR * M_PI/180.0 + p_theta;
        theta[i][1] = 0.02;

    } */
    //printf("W= %lf\n",W_sum);
    double p[3];
    for (int i=0; i< PARTICLE_NUM; i++)
    {
        // printf("[%d] x=%f y=%f z=%f\n",i, rc->particle[i].pos[0] ,rc->particle[i].pos[1] , rc->particle[i].pos[2] );
        for (int j=0; j < 3; j++)
        {
            p[j] = rc->particle[i].pos[j];
        }
        dsDrawSphere( p, R, rb);
    }

    counter++;
}


/*** [Ũf[^擾@***/
void laserReceiveData(laser_t *urg, long data[], int data_max)
{
    const dReal *pos;

    pos = dBodyGetPosition(laser.body);
    for (int i = 0; i < BEAM_NUM; i++)
    {
        if (!((g_contact_pos[i][0] == -1) && (g_contact_pos[i][1] == -1) &&
                (g_contact_pos[i][2] == -1)))
        {
            // `ɂ邽߂ɊԈ
            dReal d2 = 0;
            for (int j = 0; j < 3; j++)
            {
                d2 += (pos[j]-g_contact_pos[i][j]) * (pos[j] - g_contact_pos[i][j]);
            }
            data[i] = (long) (1000 * sqrt(d2));
            // printf("Laser dist[%d]=%.2f \n",i,laser_dist[i]);
        }
        else
        {
            data[i] = -1;
        }
    }
}

/*** PSD̃f[^擾@***/
void psdReceiveData()
{
    const dReal *pos;
    dReal data[PSD_NUM];


    for (int i = 0; i < PSD_NUM; i++)
    {
        pos = dBodyGetPosition(psd[i].body);
        if (!((g_psd_contact_pos[i][0] == -1) && (g_psd_contact_pos[i][1] == -1) &&
                (g_psd_contact_pos[i][2] == -1)))
        {
            dReal d2 = 0;
            for (int j = 0; j < 3; j++)
            {
                d2 += (pos[j]-g_psd_contact_pos[i][j]) * (pos[j] - g_psd_contact_pos[i][j]);
            }
            data[i] = (long) (1000 * sqrt(d2));
            rc->obstacle[i] = data[i];
        }
        else
        {
            data[i] = -1;
            rc->obstacle[i] = 999;
        }
        //printf("PSD dist[%d]=%.2f[mm] \n",i,data[i]);
    }
}





void laser_setSkipLines(laser_t *urg, int skip_lines)
{
    urg->skip_lines = skip_lines;
}

void laser_requestData(laser_t *urg, laser_request_type request_type, int first_index, int last_index)
{
    if ((first_index < 0)||(first_index > BEAM_NUM)|| (first_index > last_index))
    {
        printf("first index error \n");
    }
    else
    {
        urg->first_index = first_index;
    }

    if ((last_index < 0) || (last_index > BEAM_NUM) || (last_index < first_index))
    {
        printf("last index error \n");
    }
    else
    {
        urg->last_index  = last_index;
    }
}

/*** [UXLisb`(cjɐU ***/
static void swingLaser(dReal target)
{
    static dReal kp = 20.0, fmax = 100;

    dReal tmp   = dJointGetHingeAngle(laser_joint);
    dReal diff  = target - tmp;
    if (diff >=   M_PI) diff -= 2.0 * M_PI; // diff2΂菬
    if (diff <= - M_PI) diff += 2.0 * M_PI; // diff-2΂傫
    dReal u     = kp * diff;

    dJointSetHingeParam(laser_joint, dParamVel,  u);
    dJointSetHingeParam(laser_joint, dParamFMax, fmax);
}



/*** ǂ̐ ***/
// Ǌp. 0.1xPʂŐݒ͈͂-30.0`30.0.
void controlSteer(float steer)
{
    //float limit = 30.0 * M_PI/180.0;

    dReal vl = steer * M_PI/180.0 - dJointGetHinge2Angle1(joint[FRONT_LEFT]);
    if (vl >   0.1) vl =  0.1;
    if (vl < -0.1) vl = -0.1;
    vl *= 10.0;

    dReal vr = steer * M_PI/180.0 - dJointGetHinge2Angle1(joint[FRONT_RIGHT]);
    if (vr >   0.1) vr =  0.1;
    if (vr < -0.1) vr = -0.1;
    vr *= 10.0;

    dReal tmpl = dJointGetHinge2Angle1(joint[FRONT_LEFT]) * 180.0/M_PI;
    dReal tmpr = dJointGetHinge2Angle1(joint[FRONT_RIGHT]) * 180.0/M_PI;
    // printf("Steer left=%.1f  right=%.1f vl=%.1f vr=%.1f\n",tmpl, tmpr,vl ,vr);

    dJointSetHinge2Param(joint[FRONT_LEFT],dParamVel,vl);
    dJointSetHinge2Param(joint[FRONT_LEFT],dParamFMax, 2.0);
    dJointSetHinge2Param(joint[FRONT_LEFT],dParamFudgeFactor,0.1);
    dJointSetHinge2Param(joint[FRONT_RIGHT],dParamVel,vr);
    dJointSetHinge2Param(joint[FRONT_RIGHT],dParamFMax, 2.0);
    dJointSetHinge2Param(joint[FRONT_RIGHT],dParamFudgeFactor,0.1);
}

/*** Xs[h̐ ***/
// 쓮x(mm/sec). 1mmPʂŐݒ͈͂-2800`2800
void controlSpeed(int speed)
{
    double fMax = 100;  // őgN [Nm]

    // Xs[h
    if (speed >2800) speed = 2800;
    else if (speed < -2800) speed = -2800;

    double omega; // px [rad/s]
    omega = (speed /1000.0)/WHEEL_R;


    dJointSetHingeParam(joint[REAR_LEFT],   dParamVel, omega);
    dJointSetHingeParam(joint[REAR_LEFT],   dParamFMax, fMax);
    dJointSetHingeParam(joint[REAR_RIGHT], dParamVel, omega);
    dJointSetHingeParam(joint[REAR_RIGHT], dParamFMax, fMax);
}

/******* Stop֐stop() ********/
static void stop()
{
    rc->speed = 0;
}

void changeViewMode()
{
    switch(LAST_KEY)
    {
    case '2':
    {
        const dReal *pos = (const dReal *) dBodyGetPosition(base.body);
        xyz[0] = pos[0];
        xyz[1] = pos[1];
        xyz[2] = 4.0;
        hpr[0] = - (heading() * 180.0/M_PI - 90.0);
        hpr[1] = -90;
        hpr[2] = 0;
        dsSetViewpoint(xyz,hpr);
        break;
    }
    case '3':
    {
        const dReal *pos = (const dReal *) dBodyGetPosition(base.body);
        xyz[0] = pos[0];
        xyz[1] = pos[1];
        xyz[2] = 1.0;
        hpr[0] = - (heading() * 180.0/M_PI - 90.0);
        hpr[1] = 0;
        hpr[2] = 0;
        dsSetViewpoint(xyz,hpr);
        printf("Current Pos x=%f  y=%f\n",xyz[0],xyz[1]);
        break;
    }
    }
}



/*** V~[V[v ***/
static void simLoop(int pause)
{
    static double laser_angle = 0;
    static double diff = 1;

    if (!pause)
    {
        // {bg̐䕔
        controlSpeed(rc->speed);  // 쓮x(mm/sec). 1mmPʂŐݒ͈͂-2800`2800.
        controlSteer(rc->steer);  // Ǌp. 0.1xPʂŐݒ͈͂-30.0`30.0.emitLaser(); // [U[

        // ǂ̐px擾
        rc->servo_present_position =  dJointGetHinge2Angle1(joint[FRONT_LEFT]) * 180.0/M_PI;

        // WC [deg/s] ̃V~[V
        const dReal *g = dBodyGetAngularVel(base.body);
        dVector3 result;
        dBodyVectorFromWorld(base.body,g[0],g[1],g[2], result);  //{bgWnɕϊ

        // mCY̕t
        double noise = ((double) rand()/RAND_MAX) *  1.0;
        rc->sensor.gyro = noise + result[2] * 180.0/M_PI;

        // xv@[G] ̃V~[V
        const dReal *v = dBodyGetLinearVel(base.body);
        static dReal v_old[3];
        dReal ax,ay,az;
        ax =  (v[0] - v_old[0])/(STEP_SIZE * 9.80665);
        ay =  (v[1] - v_old[1])/(STEP_SIZE * 9.80665);
        az =  (v[2] - v_old[2])/(STEP_SIZE * 9.80665);
        dVector3 result2;
        dBodyVectorFromWorld(base.body,ax,ay,az,result2); //{bgWnɕϊ
        rc->sensor.acc_x = result2[0];
        rc->sensor.acc_y = result2[1];
        rc->sensor.acc_z = result2[2];
        for (int i=0; i < 3; i++) v_old[i] = v[i];

        // GR[_[̃V~[V
        dReal omega0, omega1, omega2, omega3, omega4;
        omega0 = dJointGetHingeAngleRate(joint[REAR_RIGHT]);
        omega1 = dJointGetHinge2Angle2Rate(joint[FRONT_LEFT]);
        omega2 = dJointGetHinge2Angle2Rate(joint[FRONT_RIGHT]);
        omega3 = dJointGetHingeAngleRate(joint[REAR_LEFT]);
        omega4 = dJointGetHingeAngleRate(joint[REAR_RIGHT]);

        // printf("omega1=%f\n",omega1);

        rc->sensor.enc_0 =  omega0 / (2.0*M_PI);  // [cycle/s]
        rc->sensor.enc_1 =  WHEEL_R * omega1 * 1000;  // [mm/s]
        rc->sensor.enc_2 =  WHEEL_R * omega2 * 1000;
        rc->sensor.enc_3 =  WHEEL_R * omega3 * 1000;
        rc->sensor.enc_4 =  WHEEL_R * omega4 * 1000;

        // LRF̃V~[V
        emitLaser();
        if ((laser_angle >= M_PI/4) || (laser_angle <= - M_PI/4))
        {
            diff *= -1;
        }


        // PSD̃V~[V
        emitPSD();


        // swingLaser(laser_angle += 5.0 * diff * M_PI/180.0);
        //deadReckoning();
        //navigation(vel, ang_vel);

        // ODE@ύXKvȂ
        dSpaceCollide(space,0,&nearCallback);

        psdReceiveData();
        laserReceiveData(&urg, laser_data,laser_data_max);

        dWorldStep(world, STEP_SIZE);
        //dWorldQuickStep(world, STEP_SIZE);
        dJointGroupEmpty(contactgroup);
    }


    // `
    if (VIEW_MODE) changeViewMode();


    //drawSphere2();

    drawRoboCar();
    drawWall();
    drawPSD2();

    if (g_particle_view) drawParticles();
    if (g_laser_view)    drawLaser();


    //drawCourse();
    g_epoch++;
    //if (epoch == 10) while (1) {}

}

/*** `֐̐ݒ ***/
static void setDrawStuff()
{
    fn.version = DS_VERSION;
    fn.start   = &start;
    fn.step    = &simLoop;
    fn.command = &command;
    fn.path_to_textures = "../../drawstuff/textures";
}

/*** [UXLȉ ***/
void initLaser()
{
    laser_data = (long*)malloc(sizeof(long) * laser_data_max);
    if (laser_data == NULL)
    {
        perror("malloc");
        exit(1);
    }
    laser_setSkipLines(&urg,2);
    laser_requestData(&urg, URG_MD, URG_FIRST, URG_LAST);
}


void welcomeMessage()
{
    printf("************************************************\n");
    printf(WELCOME_TEXT);
    printf("************************************************\n");
    printf("[Key Operation] \n");
    printf("1,2,3,4: View change \n");
    printf("W: Forward, S: Backward, A: Turn Left, D: Turn Right, Space: Stop\n");
    printf("P: Particle view on/off,  L: Laser beam view on/off");
}

void initSimu()
{
    if (g_epoch !=0)  {
        closeSimu();
    }

    g_epoch = 0; // JE^[̏

    // ODE֘A@ύX̕KvȂ
    setDrawStuff(); // `֐̐ݒ
    srand(time(NULL));
    dInitODE();
    world            = dWorldCreate();                   // [h̐
    space           = dHashSpaceCreate(0);         // Xy[X̐
    contactgroup = dJointGroupCreate(0);         // ڐG_O[v̐
    ground          = dCreatePlane(space,0,0,1,0); // nʂ̐
    dWorldSetGravity(world, 0, 0, -GRAVITY);              // d͉x̐ݒ
    //dWorldSetCFM(world,1e-5); // CFM̐ݒ
    dWorldSetERP(world, 0.8);                          // ERP̐ݒ 0.8

    //@암@ύX̕Kv
    makeRoboCar();   // {bg̐
    makeWall();         // ǂ̐
    initLaser(); // [UXLȉ

    //Sleep(2000);
    welcomeMessage(); // 悤\
}

void closeSimu()
{
    dJointGroupDestroy(contactgroup);  // ڐG_O[v̔j
    dSpaceDestroy(space);              // Xy[X̔j
    dWorldDestroy(world);              // [h̔j
    dCloseODE();
 }

void simulation(int width, int height)
{
    initSimu(); // V~[V̏
    dsSimulationLoop(1, 0, width,height,&fn);  // V~[V[v
    closeSimu();  // V~[V̏I
}

double dmGetTimeSimu()
{
    return g_epoch * STEP_SIZE;
}


/** ^̎Ȉʒupos[2]Ǝp({bg̐isjcourse߂֐@**/
void dmGetRealPosition(double *pos, double *course)
{
    const double *pb = dBodyGetPosition(base.body);

    pos[0] = pb[0]; // pb͐^̎Ȉʒu
    pos[1] = pb[1];
    pos[2] = pb[2];
    *course  = heading();
}

