/*** The RoboCar Simulator   2012-06-13 ***************/
/*** Copyright (c) 2012-2014 Kosei Demura  All rights reserved. **/
// simulation.h
#ifndef  _SIMULATION_H_
#define _SIMULATION_H_

#include "../common.h"
#include "drawstuff/drawstuff.h"


#ifdef _MSC_VER
#pragma warning(disable:4244 4305)  // for VC++, no precision loss complaints
#endif

#ifdef dDOUBLE
#define dsDrawBox      dsDrawBoxD
#define dsDrawSphere  dsDrawSphereD
#define dsDrawCylinder dsDrawCylinderD
#define dsDrawCapsule dsDrawCapsuleD
#define dsDrawLine      dsDrawLineD
#endif


#define GRAVITY 9.80665        // d͉x
#define WALL_PARTS_NUM 48  // 48 ǂ̐ ύX

// LRF  URG-04LX
#define BEAM_NUM           666 // [U[r[̖{ 1024
#define BEAM_LENGTH         4 // r[̓B [m]  40
#define LASER_LO_LIMIT  -120  // [UXLȋpx[degree]
#define LASER_HI_LIMIT     120 // [UXLȋpx
#define URG_FIRST               0 //  0
#define URG_LAST             666 //  1024

// ԊOZT@PSD
#define PSD_NUM  8
#define PSD_LENGTH  0.75 // 0.75 [m]

static dWorldID    world;
static dSpaceID   space;
static dGeomID   ground;
static dGeomID   wall_parts[WALL_PARTS_NUM];
static dGeomID   beam[BEAM_NUM];  // LIDAR
static dGeomID   psd_beam[PSD_NUM];       // PSD (Position Sensitive Detector)

static dJointGroupID contactgroup;
static dsFunctions fn;

MyObject wheel[NUM], base, neck, neck2, laser, psd[PSD_NUM];
dJointID joint[NUM], neck_joint, neck2_joint,laser_joint, psd_joint[PSD_NUM];

static dReal g_contact_pos[BEAM_NUM][3]; // [U[ƕ̂̌_
static dReal g_psd_contact_pos[PSD_NUM][3];
static int g_epoch = 0; // V~[Ṽ[v
long *laser_data;     // [UZTŎ擾
int   laser_data_max = BEAM_NUM;

static dReal v_r = 0.0,v_l = 0.0 ;                   // E,ԗւ̉]x [rad/s]
static dReal BASE_M    = 3.0;                        // Ԃ̎ [kg]

static int VIEW_MODE = 0;  // 0: ŒAPFǏ]
static int LAST_KEY   = '1';  // ŌɉꂽL[

dReal BASE_LX = 0.188, BASE_LY = 0.429, BASE_LZ= 0.015;   // Ԃ̒AA [m]
static const dReal  SENSOR_M  = 0.016;                // ZT̎ [kg]
static const dReal  SENSOR_R = 0.02, SENSOR_L = 0.04; // ZT̔a0.02C0.07 [m]
static const dReal  LASER_M  = 0.37;                   // [U̎ [kg]
static const dReal  LASER_R = 0.028, LASER_L = 0.087; // ZT̔aC [m]
dReal LASER_ANGLE = 0;

static const dReal PSD_LX=0.013, PSD_LY =0.03, PSD_LZ = 0.013, PSD_M = 0.01;

static const dReal  NECK_M = 0.001, NECK_R = 0.095, NECK_L = 0.06; // ̎ʁCaC
static const dReal  NECK2_M = 0.001, NECK2_R = 0.085, NECK2_L = 0.04; // ̎ʁCaC

dReal WHEEL_R = 0.03;   // ԗւ̔a 30mm ij
dReal WHEEL_W  = 0.02, WHEEL_M = 0.01; // ԗւ̕,
static double BASE_SIZE[3] = {BASE_LX, BASE_LY, BASE_LZ};
static double PSD_SIZE[3]   = {PSD_LX, PSD_LY, PSD_LZ};


typedef struct
{
    int skip_lines;
    int first_index;
    int last_index;
} laser_t;

enum laser_request_type
{
    URG_GD,
    URG_MD
};
laser_t urg;

// 0: right panel, 1:left panel, 2:back panel, 3;bar,
// 4: right wall paper, 5: left wall paper, 6: back wall paper

#define CONV 0.068
#define X_OFFSET 0.9
#define Y_OFFSET 1.0
static const dReal WALL_SIDES[WALL_PARTS_NUM][3] =
{
    //1
    {0.77, 2.83, 2.0}, // 0
    {0.56, 0.1, 2.0},  // 1
    {0.1, 2.27, 2.0},  // 2
    {1.28, 2.19, 2.0}, // 3
    {0.1, 1.14, 2.0},  // 4
    {3.08, 0.1, 2.0},  // 5
    //2
    {24.32, 6.45, 2.0}, // 6
    {1.08, 3.7, 2.0},   // 7
    {15.75, 1.50, 2.0}, // 8
    {1.17, 0.45, 2.0},  // 9
    {0.75, 0.87, 2.0},  // 10
    {0.75, 0.87, 2.0},  // 11
    {0.75, 0.87, 2.0},  // 12
    {6.24, 1.18, 2.0},  // 13
    //3
    {2.58, 13.73, 2.0}, // 14
    {0.1, 10.62, 2.0},  // 15
    {3.0, 3.33, 2.0},   // 16
    //4
    {4.07, 1.2, 2.0},   // 17
    {3.36, 1.2, 2.0},   // 18
    {1.12, 1.13, 2.0},  // 19
    {2.13, 0.1, 2.0},   // 20
    {4.24, 1.13, 2.0},  // 21
    //5
    {6.0, 0.1, 2.0},    // 22
    {0.78, 2.72, 2.0},  // 23
    {0.1, 3.26, 2.0},   // 24
    {0.78, 4.79, 2.0},  // 25
    {0.1, 3.26, 2.0},   // 26 wall 3.26 Goal
    {0.78, 2.41, 2.0},  // 27
    {1.84, 0.1, 2.0},   // 28
    {0.1, 16.44, 2.0},  // 29
    {20.0, 0.1, 2.0},   // 30
    //6
    {0.6, 45.61, 2.0},  // 31
    {0.75, 18.75, 2.0}, // 32
    {0.1, 3.25, 2.0},   // 33
    {0.75, 4.77, 2.0},  // 34
    {0.1, 3.25, 2.0},   // 35
    {0.75, 4.77, 2.0},  // 36
    {0.1, 3.25, 2.0},   // 37
    {2.7, 2.6, 2.0},    // 38
    //7
    {1.5, 0.1, 2.0},    // 39
    {0.1, 1.2, 2.0},    // 40
    {4.1, 5.9, 2.0},    // 41
    {0.1, 7.6, 2.0},    // 42
    {2.25, 0.1, 2.0},   // 43

    //8
    {2.45, 0.1, 2.0},   // 44
    {0.1, 0.33, 2.0},   // 45
    {2.45, 0.1, 2.0},   // 46
    {0.1, 2.83, 2.0},   // 47


};

static const dReal FIELD_SIDES[3] = {10,14,0};

//extern void deadReckoning();
//|ύX
//Ȉʒux,y,thetaԂĂ

//extern void deadReckoning( dReal *x, dReal *y, dReal *theta);

//KEXzɂx,y,thetaԂ
double gaussrand(double *x, double *y, double *theta);
//ZTf[^
void getPos(double *sensor_pos_x, double *sensor_pos_y, double *sensor_course);

void simulation(int width, int height);
void initSimu(); // V~[V
void closeSimu(); // V~[VI

double dmGetTimeSimu(); // V~[V̎(b)
void dmGetRealPosition(double *pos, double *course);
#endif
